# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk  # pylint: disable=E0611
from gi.repository import Gio  # pylint: disable=E0611

from diabetes_tracker_lib.helpers import get_builder
import datetime
import sqlite3
import gettext
import os
from gettext import gettext as _
gettext.textdomain('diabetes-tracker')

CONVERSION_FACTOR = 18.0182
DATABASE_LOCATION = os.getenv("HOME") + "/.cache/diabetes-tracker/diabetes-tracker.db"


class AddreadingDialog(Gtk.Dialog):
    __gtype_name__ = "AddreadingDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when
        constructing a new instance of this class.

        Returns a fully instantiated AddreadingDialog object.
        """
        builder = get_builder('AddreadingDialog')
        new_object = builder.get_object('addreading_dialog')
        new_object.finish_initializing(builder)

        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a AddreadingDialog object with it in order to
        finish initializing the start of the new AddreadingDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)

        # Test to see if the day or month comes first in the locale
        # test date is 25 December 2012 at midnight
        test_time = datetime.datetime.fromtimestamp(1356393600)
        test_time = test_time.strftime("%x")
        if test_time[3:5] == "12":
            proper_time = True
        else:
            proper_time = False

        self.settings = Gio.Settings("net.launchpad.diabetes-tracker")

        self.spinbuttonday = self.builder.get_object("spinbuttonday")
        self.spinbuttonmonth = self.builder.get_object("spinbuttonmonth")
        self.spinbuttonyear = self.builder.get_object("spinbuttonyear")
        self.spinbuttonhour = self.builder.get_object("spinbuttonhour")
        self.spinbuttonminute = self.builder.get_object("spinbuttonminute")
        self.spinbuttonreading = self.builder.get_object("spinbuttonreading")
        self.spinbuttonreadinglabel = self.builder.get_object("label3")
        self.box3 = self.builder.get_object("box3")

        if not proper_time:
            self.box3.reorder_child(self.spinbuttonmonth, 1)

        def show_leading_zeros(spin_button):
            adjustment = spin_button.get_adjustment()
            spin_button.set_text('{:02d}'.format(int(adjustment.get_value())))
            return True

        self.spinbuttonminute.connect('output', show_leading_zeros)
        now = datetime.datetime.now()
        self.spinbuttonday.set_value(now.day)
        self.spinbuttonmonth.set_value(now.month)
        self.spinbuttonyear.set_value(now.year)
        self.spinbuttonhour.set_value(now.hour)
        self.spinbuttonminute.set_value(now.minute)

        # True means use mmol/l, False means mg/dl
        if self.settings.get_boolean("units") is True:
            self.spinbuttonreading.set_digits(1)
            self.spinbuttonreading.set_increments(0.1, 1)
            self.spinbuttonreading.set_value(5)
            self.spinbuttonreadinglabel.set_text("mmol/l")
        else:
            self.spinbuttonreading.set_digits(0)
            self.spinbuttonreading.set_increments(1, 1)
            self.spinbuttonreading.set_value(100)
            self.spinbuttonreadinglabel.set_text("mg/dl")

    def on_btn_ok_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.ResponseType.OK from run().
        """
        day = self.spinbuttonday.get_value()
        month = self.spinbuttonmonth.get_value()
        year = self.spinbuttonyear.get_value()
        hour = self.spinbuttonhour.get_value()
        minute = self.spinbuttonminute.get_value()
        selecteddate = "%s %s %s %s %s" % (int(day), int(month), int(year), int(hour), int(minute))
        timestamp = datetime.datetime.strptime(selecteddate, "%d %m %Y %H %M")
        timestamp = timestamp.strftime("%s")
        reading = self.spinbuttonreading.get_value()

        if self.settings.get_boolean("units") is False:
            reading = reading / CONVERSION_FACTOR

        # Add the reading to the database
        conn = sqlite3.connect(DATABASE_LOCATION)
        cursor = conn.cursor()
        cursor.execute('INSERT INTO glucose_readings VALUES(%s, %s)' % (timestamp, reading))
        conn.commit()
        cursor.close()

        self.destroy()

    def on_btn_cancel_clicked(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns Gtk.ResponseType.CANCEL for run()
        """
        self.destroy()

if __name__ == "__main__":
    dialog = AddreadingDialog()
    dialog.show()
    Gtk.main()
