# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from locale import gettext as _

from gi.repository import Gtk  # pylint: disable=E0611
from gi.repository import Gio  # pylint: disable=E0611
import logging
logger = logging.getLogger('diabetes_tracker')

from diabetes_tracker_lib import Window
from diabetes_tracker.AboutDiabetesTrackerDialog import AboutDiabetesTrackerDialog
from diabetes_tracker.PreferencesDiabetesTrackerDialog import PreferencesDiabetesTrackerDialog
from diabetes_tracker.AddreadingDialog import AddreadingDialog

import matplotlib
matplotlib.use('SVG')
import matplotlib.pyplot as plt
import matplotlib.dates as dates
from matplotlib.ticker import MaxNLocator
import datetime
import sqlite3
import os

# mmol/l * 18.0182 = mg/dl for blood glucose
CONVERSION_FACTOR = 18.0182
GRAPH_SVG_LOCATION = os.getenv("HOME") + "/.cache/diabetes-tracker/graph.svg"
DATABASE_LOCATION = os.getenv("HOME") + "/.cache/diabetes-tracker/diabetes-tracker.db"


# See diabetes_tracker_lib.Window.py for more details about how this class works
class DiabetesTrackerWindow(Window):
    __gtype_name__ = "DiabetesTrackerWindow"

    def finish_initializing(self, builder):  # pylint: disable=E1002
        """Set up the main window"""
        super(DiabetesTrackerWindow, self).finish_initializing(builder)
        if not os.path.exists(os.getenv("HOME") + "/.cache/diabetes-tracker"):
            os.makedirs(os.getenv("HOME") + "/.cache/diabetes-tracker")
        self.AboutDialog = AboutDiabetesTrackerDialog
        self.PreferencesDialog = PreferencesDiabetesTrackerDialog
        self.AddreadingDialog = AddreadingDialog

        self.settings = Gio.Settings("net.launchpad.diabetes-tracker")
        
        self.notebook1 = self.builder.get_object("notebook1")
        self.box1 = self.builder.get_object('box1')
        self.box2 = self.builder.get_object('box2')
        self.image = self.builder.get_object("image1")
        self.a1c = self.builder.get_object("label2")
        self.abg = self.builder.get_object("label4")
        self.button1 = self.builder.get_object('radiobutton1')
        self.button2 = self.builder.get_object('radiobutton2')
        self.liststore = self.builder.get_object('liststore1')
        self.treeview = self.builder.get_object('treeview1')
        self.toolbar = self.builder.get_object("toolbar1")
        self.highentry = self.builder.get_object("high-entry")
        self.lowentry = self.builder.get_object("low-entry")
        
        self.lowdate = datetime.datetime.now() - datetime.timedelta(weeks=3, days=6)
        self.highdate = datetime.datetime.now() + datetime.timedelta(days=1)

        context = self.toolbar.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
        self.setup_database()
        # Code for other initialization actions should be added here.
        self.draw()

    def setup_database(self):
        # setup the database if it doesn't exist
        if not os.path.exists(DATABASE_LOCATION):
            conn = sqlite3.connect(DATABASE_LOCATION)
            cursor = conn.cursor()
            cursor.execute('CREATE TABLE glucose_readings(time INT, reading INT)')
            conn.commit()
            cursor.close()
        # setup a dummy graph svg file if one doesn't exist
        if not os.path.exists(GRAPH_SVG_LOCATION):
            plt.ylabel("Reading (mmol/l)")
            x1, x2, y1, y2 = plt.axis()
            plt.axis((x1, x2, 0, 10))

            fig = matplotlib.pyplot.gcf()
            fig.set_size_inches(6, 4)
            plt.savefig(GRAPH_SVG_LOCATION, dpi=100)

    def add_reading(self, type1):
        """When Add reading is clicked, the Add Reading Dialog is shown"""
        self.AddreadingDialog.run(AddreadingDialog())
        self.draw()

    def draw(self):
        #if not self.lowdate:
        #    self.lowdate = datetime.datetime.now() - datetime.timedelta(weeks=4)
        #if not highdate:
        #    self.highdate = datetime.datetime.now()

        query = 'SELECT * FROM glucose_readings WHERE time > %s AND time < %s ORDER BY time DESC' % (self.lowdate.strftime("%s"), self.highdate.strftime("%s"))
        
        conn = sqlite3.connect(DATABASE_LOCATION)
        cursor = conn.cursor()
        cursor.execute(query)
        readings = cursor.fetchall()
        cursor.close()
        if readings:
            self.draw_plot(readings)
            self.draw_list(readings)
            self.notebook1.show()
            self.box1.show()
            self.box2.hide()
        else:
            self.notebook1.hide()
            self.box1.hide()
            self.box2.show()
            if self.settings.get_boolean("units") is True:
                self.button1.set_active(True)
            else:
                self.button2.set_active(True)
            self.button1.connect('toggled', self.on_units_toggled, 'radiobutton1')
            print "No readings"
    
    def on_units_toggled(self, a, b):
        if self.button1.get_active() is True:
            self.settings.set_boolean("units", True)
        else:
            self.settings.set_boolean("units", False)

    def draw_plot(self, readings):
        font = {'family': 'ubuntu',
                'weight': 'normal',
                'size': 12}

        matplotlib.rc('font', **font)
        xplot = []
        yplot = []
        plt.gcf().clear()
        conversion_factor = 1
        units = "mmol/l"
        if self.settings.get_boolean("units") is False:
            conversion_factor = CONVERSION_FACTOR
            units = "mg/dl"
        for reading in readings:
            xplot.append(datetime.datetime.fromtimestamp(reading[0]))
            yplot.append(conversion_factor * reading[1])
        abgplot = []
        maxy = max(yplot)
        if maxy < 9 * conversion_factor:
            maxy = 9 * conversion_factor
        for point in range(len(yplot)):
            abgplot.append(sum(yplot) / float(len(yplot)))
        plt.plot_date(xplot, yplot, "r-", linewidth=3.0, marker='o')
        plt.plot_date(xplot, abgplot, "r:", linewidth=1.0)

        low = self.settings.get_double("target-range-low") * conversion_factor
        high = self.settings.get_double("target-range-high") * conversion_factor
        if high > maxy + (1 * conversion_factor):
            high = maxy + (1 * conversion_factor)

        plt.axhspan(low, high, color="#ccffcc")
        plt.gcf().axes[0].xaxis.set_major_formatter(dates.DateFormatter('%x'))
        plt.gcf().axes[0].xaxis.set_major_locator(MaxNLocator(5))
        plt.ylabel("Reading (%s)" % units)

        x1, x2, y1, y2 = plt.axis()
        plt.axis((self.lowdate, self.highdate, 0, maxy + (1 * conversion_factor)))

        fig = matplotlib.pyplot.gcf()
        fig.set_size_inches(6, 4)
        plt.savefig(GRAPH_SVG_LOCATION, dpi=100)

        self.image.set_from_file(GRAPH_SVG_LOCATION)

        # Set up the graph date controls
        graph_low_value = xplot[len(xplot)-1]
        graph_high_value = xplot[0]
        self.lowentry.set_text(self.lowdate.strftime("%x"))
        self.highentry.set_text(self.highdate.strftime("%x"))

        # Set the labels for the average blood glucose and estimated HbA1c
        abg = sum(yplot) / float(len(yplot))
        self.abg.set_label(str(round(abg, 1)) + " %s" % units)

        # Estimated HbA1c (in the old % format) based on average bloog glucose (abg (in mg/dl)) is
        # a1c  = (abg + 46.7) / 28.7
        # HbA1c in the new mmol/mol format from the old % format is
        # mmola1c = (%a1c - 2.15) * 10.929
        a1c = ((((abg * 18.0182 / conversion_factor) + 46.7) / 28.7) - 2.15) * 10.929
        self.a1c.set_label(str(int(a1c)) + " mmol/mol")

    def change_range(self, high_or_low, up_or_down):
        lowvalue = self.lowentry.get_text()
        highvalue = self.highentry.get_text()
        lowtime = datetime.datetime.strptime(lowvalue, "%x")
        hightime = datetime.datetime.strptime(highvalue, "%x")
        
        if high_or_low == "high":
            if up_or_down == "up":
                print "High up"
                hightime = hightime + datetime.timedelta(weeks=1)
            else:
                print "High down"
                hightime = hightime - datetime.timedelta(weeks=1)
        else:
            if up_or_down == "up":
                print "Low up"
                lowtime = lowtime + datetime.timedelta(weeks=1)
            else:
                print "Low down"
                lowtime = lowtime - datetime.timedelta(weeks=1)

        self.lowentry.set_text(lowtime.strftime("%x"))
        self.highentry.set_text(hightime.strftime("%x"))
        self.lowdate = lowtime
        self.highdate = hightime
        self.draw()
        
    def reduce_lowdate(self, a):
        self.change_range("low", "down")
        
    def increase_lowdate(self, a):
        self.change_range("low", "up")
        
    def reduce_highdate(self, a):
        self.change_range("high", "down")
        
    def increase_highdate(self, a):
        self.change_range("high", "up")

    def draw_list(self, readings):
        conversion_factor = 1
        if self.settings.get_boolean("units") is False:
            conversion_factor = CONVERSION_FACTOR
        self.liststore.clear()
        for reading in readings:
            if conversion_factor == 1:
                glucose_reading = round(reading[1], 1)
            else:
                glucose_reading = int(conversion_factor * reading[1])
            datestring = datetime.datetime.fromtimestamp(reading[0])
            datestring = datestring.strftime("%x\t %H:%M")
            self.liststore.append([str(datestring), str(glucose_reading), str(reading[0])])

    def on_row_activated(self, a, row, column):
        if column.get_name() == "delete":
            time = self.liststore.get_value(self.liststore.get_iter(row), 2)

            conn = sqlite3.connect(DATABASE_LOCATION)
            cursor = conn.cursor()
            cursor.execute('DELETE FROM glucose_readings WHERE time = "%s"' % time)
            conn.commit()
            cursor.close()
            self.draw()

    def open_preferences(self, type1):
        """When Add reading is clicked, the Add Reading Dialog is shown"""
        self.PreferencesDialog.run(PreferencesDiabetesTrackerDialog())
        self.draw()
